import datetime


class SoilWaterBalance(object):
    """
    class water_balance.SoilWaterBalance:
        Calculates soil water balance. For more details, <link>

        Attributes:
            fc: The field capacity [mm]
            wp: The wilting point [cm]
            rd: The crop root depth [cm]
            kc: The crop coefficient
            p: The crop depletion fraction
            precipitation: A daily Timeseries object [mm]
            evapotranspiration:  A daily Timeseries object [mm]
            rd_factor: root depth conversion fraction
            taw: Total available water, [m3m-3]
            raw: Readily available water,  [m3m-3]
            irrigation_efficiency: The irrigation efficiency factor

        Methods:
            root_zone_depletion:
                Calculates root zone depletion
                Inputs:
                    start_date: A datetime object
                    initial_soil_moisture: Initial Soil Moisture [mm]
                    end_date: A datetime object
                Output:
                    depletion: root zone depletion for end_date [mm]

            irrigation_water_amount:
                Calculates irrigation water needs based in irrigation method
                        efficiency
                Inputs:
                    start_date: A datetime object
                    initial_soil_moisture: Initial Soil Moisture [mm]
                    end_date: A datetime object
                Output:
                    irrigation_amount: irrigation water needs for end_date [mm]

    """

    def __init__(self, fc, wp, rd, kc, p, precipitation,
                 evapotranspiration, irrigation_efficiency, rd_factor=1):
        self.fc = fc
        self.wp = wp
        self.rd = rd
        self.kc = kc
        self.p = p
        self.precip = precipitation
        self.evap = evapotranspiration
        self.rd_factor = rd_factor
        self.taw = (self.fc - self.wp) * self.rd
        self.raw = self.p * self.taw
        self.irrigation_efficiency = irrigation_efficiency

    def root_zone_depletion(self, start_date, initial_soil_moisture, end_date):
        depletion = (initial_soil_moisture * self.rd * self.rd_factor) / self.fc
        delta = datetime.timedelta(days=1)
        day = start_date + delta
        while day <= end_date:
                depletion = depletion - self.precip[day] + self.kc * self.evap[day]
                if depletion < 0:
                    depletion = 0
                day += delta
        return depletion

    def irrigation_water_amount(self, start_date, initial_soil_moisture,
                                end_date):
        depletion = self.root_zone_depletion(start_date, initial_soil_moisture,
                                             end_date)
        irrigation_water_amount = depletion / self.irrigation_efficiency
        return irrigation_water_amount



from datetime import datetime
import textwrap
from unittest import TestCase

from six import StringIO

from pthelma.timeseries import Timeseries, TimeStep
from pthelma.water_balance import SoilWaterBalance


precipitation_test_timeseries = textwrap.dedent("""\
            2014-11-01,0,
            2014-11-02,0,
            2014-11-03,0,
            2014-11-04,10,
            2014-11-05,7,
            2014-11-06,4,
            2014-11-07,0,
            2014-11-08,2,
            2014-11-09,1,
            2014-11-10,0,
            """)

evapotranspiration_test_timeseries = textwrap.dedent("""\
            2014-11-01,0,
            2014-11-02,0,
            2014-11-03,0,
            2014-11-04,2,
            2014-11-05,1.4,
            2014-11-06,0.8,
            2014-11-07,0,
            2014-11-08,0.4,
            2014-11-09,0.2,
            2014-11-10,0,
            """)


class SoilWaterBalanceTestCase(TestCase):

    def test_root_zone_depletion(self):
        # pthelma/tests/data/SoilWaterBalance_calculations.obs
        precipitation = Timeseries(time_step=TimeStep(1440, 0))
        instring = StringIO(precipitation_test_timeseries)
        precipitation.read_file(instring)

        evapotranspiration = Timeseries(time_step=Timeseries(1440, 0))
        instring = StringIO(evapotranspiration_test_timeseries)
        evapotranspiration.read_file(instring)

        swb = SoilWaterBalance(0.5, 1, 0.5, 0.75,
                               1, precipitation, evapotranspiration, 1.2, 1)
        depletion = swb.root_zone_depletion(datetime(2014, 11, 1), 100,
                                            datetime(2014, 11, 10))
        self.assertAlmostEqual(depletion, 79.6)

    def test_irrigation_water_amount(self):
        # pthelma/tests/data/SoilWaterBalance_calculations.obs
        precipitation = Timeseries(time_step=TimeStep(1440, 0))
        instring = StringIO(precipitation_test_timeseries)
        precipitation.read_file(instring)

        evapotranspiration = Timeseries(time_step=Timeseries(1440, 0))
        instring = StringIO(evapotranspiration_test_timeseries)
        evapotranspiration.read_file(instring)

        swb = SoilWaterBalance(0.5, 1, 0.5, 0.75, 1,
                               precipitation, evapotranspiration, 1.2, 1)
        iwa = swb.irrigation_water_amount(datetime(2014, 11, 1),
                                          100, datetime(2014, 11, 10))
        self.assertAlmostEqual(iwa, 66.3333333)
